<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Bandcamp Electronic Music Library Classifier - Admin</title>
    <link rel="stylesheet" href="static/css/styles.css">
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@4.5.3/dist/css/bootstrap.min.css"
        integrity="sha384-TX8t27EcRE3e/ihU7zmQxVncDAy5uIKz4rEkgIXeMed4M0jlfIDPvg6uqKI2xXr2" crossorigin="anonymous">
</head>

<body>

       
    <div class="homepage-container">

        <h1>Bandcamp Dance Librarian - Admin</h1>

        <hr style="height:20px;border-width:0;color:gray;background-color:gray">

        <br><br>

        <p class="classifier-text">Add another electornic dance music bandcamp label/library to the database.</p>
        <p class="description">The library should contain at least 100 tracks to be listed on the main page.</p>
        <p class="description">The algorithm will process the tracks individually and extract their audio features.</p>
        <p class="description">You can also add a pre-existing library for updating it with recently released tracks.</p>   
        <form id="inputform" action="/addlibrary">
            <p class="description">https://<input type="text" id="libname" name="libname">.bandcamp.com</p>
            <p style="text-align: center">
            <input type="submit" id="button1" name="submit" value="Submit label" class="btn btn-primary btn-lg" {{disabled}}>
            </p>
        </form>

        <p class="warning" id="wrn">{{ warning_message }}</p>
    
    <p></p>

        <hr style="height:20px;border-width:0;color:gray;background-color:gray">
        
        <p class="classifier-text">Download the tracks database in CSV format.</p>
        <form id="dbdump" action="/gettracks">
        <button class="btn btn-primary btn-lg">Get database</button></a>
        </form>

        <hr style="height:20px;border-width:0;color:gray;background-color:gray">
        
        <p class="classifier-text">Download the shortlist of labels with the number of tracks processed in CSV format.</p>
        <form id="dbdump" action="/getlabels">
        <button class="btn btn-primary btn-lg">Get label list</button></a>
        </form>

    </div>

    <!-- Bootstrap JS -->
    <script src="https://code.jquery.com/jquery-3.5.1.slim.min.js"
        integrity="sha384-DfXdz2htPH0lsSSs5nCTpuj/zy4C+OGpamoFVy38MVBnE+IbbVYUew+OrCXaRkfj"
        crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.1/dist/umd/popper.min.js"
        integrity="sha384-9/reFTGAW83EW2RDu2S0VKaIzap3H66lZH81PoYlFhbGU+6BZp6G7niu735Sk7lN"
        crossorigin="anonymous"></script>
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.5.3/dist/js/bootstrap.min.js"
        integrity="sha384-w1Q4orYjBQndcko6MimVbzY0tgp4pWB4lZ7lr30WKz0vr/aWKhXdBNmNb5D92v7s"
        crossorigin="anonymous"></script>

</body>
</html>

<!-- HTML COMMENTS  -->