body {
   background-color: #b9faf8 !important;
}

h1 {
  color: #333;
  padding: 15px;
}

p.classifier-text {
  color: ##227c9d;
  font-style: italic;
  font-weight: bold;
}

p.warning {
  color: #FF00AF;
  font-weight: bold;
}

p.description {
  color: #333333;
}

p.button-container {
  display: flex;
  flex-direction: column;
  justify-content: center;
  color: #006ba6;
}

ul li {
 font-size: 16px;
}

.homepage-container {
  background-color: #b9faf8;
  color: #227c9d;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
  padding: 20px 10px 20px 10px;
}

.btn {
  margin-top: 10px;
}
